#include <cstdarg>
#include <vector>

using namespace std;

vector<unsigned> array(unsigned a,unsigned b,unsigned c,unsigned d)
{
    vector<unsigned> result(4);
    result[0]=a;result[1]=b;result[2]=c;result[3]=d;
    return result;
}

struct elem
{
    int integer;        /* Number this is multiplied by      */
    unsigned exponent;  /* 0=constant, 1=x, 2=xx, 3=xxx, ... */
public:
    elem() : integer(0), exponent(0) { }
    elem(int add, unsigned exp) : integer(add), exponent(exp) { }
};


static bool TestGuess(
    const vector<unsigned> &keys,
    const vector<unsigned> &values,
    const vector<elem> &guess)
{
    bool wasok = true;
    
    /* Test the guess */

    for(unsigned a=0; a<keys.size(); ++a)
    {
        int val = keys[a];
        int result = 0;
            
        for(unsigned b=0; b<guess.size(); ++b)
        {
            int thisval = 1;
            for(unsigned c=0; c<guess[b].exponent; ++c)thisval *= val;
            thisval *= guess[b].integer;
            result += thisval;
        }
        if(result != (int)values[a])
        {
            /* Wrong */
            //wasok = false;
            //fprintf(stderr, "  Nope: "); fflush(stderr);
            //goto Ouch;
            return false;
        }
    }
    fprintf(stderr, "\nFound:  "); fflush(stderr);

Ouch:
    for(unsigned b=0; b<guess.size(); ++b)
    {
        if(guess[b].exponent == 0)
        {
            if(guess[b].integer)
            {
                if(b) printf(" %c %d", guess[b].integer<0 ? '-' : '+', abs(guess[b].integer));
                else printf(" %d", guess[b].integer);
            }
        }    
        else if(guess[b].integer)
        {
            if(guess[b].integer == -1)printf(" -");
            else if(guess[b].integer != 1)
                printf(" %+d", guess[b].integer);
            else { if(b) printf(" + "); }
            for(unsigned c=0; c<guess[b].exponent; ++c)putchar('x');
        }
    }
    printf("\n");
    fflush(stdout);
    
    if(wasok) { fprintf(stderr, "\n"); fflush(stderr); }
    
    return wasok;
}

static int calcdet(const vector<int> &mat, unsigned depf, bool sub=false)
{
    if(!sub)
        for(unsigned a=0; a<depf; ++a)
        {
            fprintf(stderr, "             ");
            for(unsigned b=0; b<depf; ++b)
                fprintf(stderr, "%-5d", mat[a*depf+b]);
            fprintf(stderr, "\n");
        }
    int sum;
    if(depf <= 1)
        sum = mat[0];
    else
    {
        sum = 0;
        for(unsigned a=0; a<depf; ++a)
        {
            unsigned subdepf = depf-1;
            vector<int> tmp(subdepf * subdepf);
            for(unsigned b=0; b<subdepf; ++b)
                for(unsigned c=0; c<subdepf; ++c)
                    tmp[b*subdepf + c] = mat[(1+b) * depf + (c + (c>=a))%depf];
            int subdet = mat[a] * calcdet(tmp, subdepf, true);
            if(a & 1)
                sum -= subdet;
            else
                sum += subdet;
        }
    }
    if(!sub)
    {
        fprintf(stderr, "             -> det: %d\n", sum);
    }
    fflush(stderr);
    fflush(stdout);
    return sum;
}

static void Guess(
    const vector<unsigned> &keys,
    const vector<unsigned> &values)
{
    if(keys.size() != values.size())
    {
        fprintf(stderr, "Error: keys.size() != values.size()\n");
        return;
    }
    
    for(unsigned elemcount=1; elemcount<=keys.size(); ++elemcount)
    {
        vector<int> mat (elemcount * elemcount);
        
        for(unsigned a=0; a<elemcount; ++a)
            for(unsigned b=0; b<elemcount; ++b)
            {
                unsigned val = 1;
                for(unsigned c=0; c<b; ++c)val *= keys[a];
                mat[a*elemcount + b] = val;
            }
        
        int det = calcdet(mat, elemcount, true);
        
        vector<elem> guess(elemcount);
        
        for(unsigned a = 0; a < elemcount; ++a)
        {
            vector<int> mat2 = mat;
            
            for(unsigned c = 0; c < elemcount; ++c)
                mat2[c*elemcount+a] = values[c];
            
            guess[elemcount-1-a].integer  = calcdet(mat2, elemcount, true) / det;
            guess[elemcount-1-a].exponent = a;
        }
        
        if(TestGuess(keys, values, guess))
            return;
    }
    fprintf(stderr, "Not found!\n");
    fflush(stderr);
}

static void Testmode()
{
    /* Should output: x + 4 */     
    
    Guess( array(0,1,2,3),           /*    x*0 + y*1 = 4      x=(4*1-0*5) / (1-0)=4 */
           array(4,5,6,7) );         /*    x*1 + y*1 = 5      y=(5  -  4) / (1-0)=1 */
    
    /* Should output: 2x */

    Guess( array(10,11,12,13),       /*    x*10 + y*1 = 20    x=(20*11-22*10) / (11-10)=0 */
           array(20,22,24,26) );     /*    x*11 + y*1 = 22    y=(22   -20)    / (11-10)=2 */
    
    /* Should output: 3x + 1 */
    
    Guess( array(1,2,3,4),
           array(4,7,10,13) );
    
    /* Should output: xx - 1 */
    
    Guess( array(1,2,3,4),
           array(0,3,8,15) );

    /* Should output: xx - 2x +20 */
    
    Guess( array( 1, 2, 3, 4),    /*  x*1 + y*1 + z*1 = 19  x = 1*(2-3)+1(-9blah  */
           array(19,20,23,28) );  /*  x*4 + y*2 + z*1 = 20  it works anyway. :) */
                                  /*  x*9 + y*3 + z*1 = 23 */
}

#include "argh.hh"
int main(void)
{
	Testmode();
  
    return 0;
}
